/*********************************************************
*Copyright (C) 2004 Neil Horman
*This program is free software; you can redistribute it and\or modify
*it under the terms of the GNU General Public License as published 
*by the Free Software Foundation; either version 2 of the License,
*or  any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*GNU General Public License for more details.
*
*File: listener.c
*
*Author:Neil Horman
*
*Date: Feb 2, 2005
*
*Description: Listens on sctp connections and rcv. data
*********************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>


int *sds = NULL;
int listen_sock;
int num_socks = 0;


void cleanup(int exitcode)
{
	int i;
	if(listen_sock > 0)
		close(listen_sock);

	for(i=0;i<num_socks;i++) {
		if(sds[i] > 0)
			close(sds[i]);
	}
	free(sds);
	exit(exitcode);
}

void sigint_handler(int sig, siginfo_t *info, void *unused)
{
	printf("Closing on SIGINT\n");
	cleanup(0);
}

void receive_data()
{
	int i;
	unsigned char rcv_byte;
	size_t rc;

	while(1) {
		for(i=0;i<num_socks;i++) {
			rcv_byte = 0;
			rc = recv(sds[i],&rcv_byte,sizeof(rcv_byte),0);
			if(rc < 0) {
				perror("Could not receive data");
				cleanup(5);
			} else if (rc == 0) {
				printf("no data returned from recv\n");
				cleanup(6);
			}
			printf("Recv byte 0x%02x on socket %d\n",rcv_byte,i);
		}
	}
}

int main(int argc, char **argv)
{
	int i;
	struct sockaddr_in conn_addr;
	int rc;
	struct sigaction sigint_action;
	socklen_t addr_len;
	unsigned int flags;

	sigint_action.sa_sigaction = sigint_handler;
	sigemptyset(&sigint_action.sa_mask);
	sigint_action.sa_flags = SA_SIGINFO;
	sigaction(SIGINT,&sigint_action,NULL);

	num_socks = atoi(argv[1]); 
	sds = malloc(sizeof(int)*num_socks);
	conn_addr.sin_family = PF_INET;
	conn_addr.sin_addr.s_addr = INADDR_ANY; 
	conn_addr.sin_port = atoi(argv[3]);
	listen_sock = socket(PF_INET,SOCK_STREAM,IPPROTO_SCTP);
	if(listen_sock < 0) {
		perror("Unable to create listen socket");
		cleanup(1);
	}
	if(bind(listen_sock,(struct sockaddr *)&conn_addr,
		sizeof(struct sockaddr_in)) < 0) {
		perror("Unable to bind socket");
		cleanup(2);
	}
	if(listen(listen_sock,num_socks) < 0) {
		perror("Unable to set socket to listen");
		cleanup(3);
	}
	
	for(i=0;i<num_socks;i++) {
		sds[i] = accept(listen_sock,(struct sockaddr *)&conn_addr,
				&addr_len);
		if(sds[i] < 0) {
			perror("Error in accept");
			cleanup(4);
		}

		flags = fcntl(sds[i],F_GETFL);
		flags &= ~(O_NONBLOCK);
		fcntl(sds[i],F_SETFL,flags);	
	}

	receive_data();

	return 0;
}
