/*********************************************************
*Copyright (C) 2004 Neil Horman
*This program is free software; you can redistribute it and\or modify
*it under the terms of the GNU General Public License as published 
*by the Free Software Foundation; either version 2 of the License,
*or  any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*GNU General Public License for more details.
*
*File: sender.c
*
*Author:Neil Horman
*
*Date: Feb 2, 2005
*
*Description: creates sctp connections and sends data.
*********************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>


int *sds = NULL;
int num_socks = 0;


void cleanup(int exitcode)
{
	int i;
	for(i=0;i<num_socks;i++) {
		if(sds[i] > 0)
			close(sds[i]);
	}
	free(sds);
	exit(exitcode);
}

void sigint_handler(int sig, siginfo_t *info, void *unused)
{
	printf("Closing on SIGINT\n");
	cleanup(0);
}

void send_data()
{
	int i;
	unsigned char send_byte = 0xab;
	int rc;

	while(1) {
		for(i=0;i<num_socks;i++) {
			rc = send(sds[i],&send_byte,sizeof(send_byte),0);
			if(rc < 0) {
				perror("Could not send data");
				cleanup(3);
			} else if (rc == 0) {
				printf("No data sent\n");
				cleanup(4);
			}
			printf("Sent byte 0x%02x on socket %d\n",send_byte,i);
		}
	}
}

int main(int argc, char **argv)
{
	int i;
	struct sockaddr_in conn_addr;
	int rc;
	struct sigaction sigint_action;
	int flags;

	sigint_action.sa_sigaction = sigint_handler;
	sigemptyset(&sigint_action.sa_mask);
	sigint_action.sa_flags = SA_SIGINFO;
	sigaction(SIGINT,&sigint_action,NULL);

	num_socks = atoi(argv[1]); 
	sds = malloc(sizeof(int)*num_socks);
	conn_addr.sin_family = PF_INET;
	conn_addr.sin_addr.s_addr = inet_addr(argv[2]);
	conn_addr.sin_port = atoi(argv[3]);
	for(i=0;i<num_socks;i++) {
		sds[i] = socket(PF_INET,SOCK_STREAM,IPPROTO_SCTP);
		if(sds[i] < 0) {
			perror("Unable to create socket");
			cleanup(1);
		}

		rc = connect(sds[i],(struct sockaddr *)&conn_addr,
				sizeof(struct sockaddr_in));
		if(rc < 0) {
			perror("Unable to connect socket");
			cleanup(2);
		}
		
		flags = fcntl(sds[i],F_GETFL);
		flags &= ~(O_NONBLOCK);
		fcntl(sds[i],F_SETFL,flags);
		
	}

	send_data();

	return 0;
}
